<?php
/**
 * Exif Caption
 * 
 * @package    Exif Caption
 * @subpackage ExifCaptionAdmin Main & Management screen
/*  Copyright (c) 2015- Katsushi Kawamori (email : dodesyoswift312@gmail.com)
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

$exifcaptionadmin = new ExifCaptionAdmin();

class ExifCaptionAdmin {

	/* ==================================================
	 * Construct
	 * @since	2.07
	 */
	public function __construct() {

		if (!class_exists('ExifCaption')){
			require_once( dirname(__FILE__).'/ExifCaption.php' );
		}

		add_filter( 'plugin_action_links', array($this, 'settings_link'), 10, 2 );
		add_action( 'admin_menu', array($this, 'add_pages'));
		add_action( 'admin_enqueue_scripts', array($this, 'load_custom_wp_admin_style') );
		add_filter( 'manage_media_columns', array($this, 'muc_column') );
		add_action( 'manage_media_custom_column', array($this, 'muc_value'), 10, 2 );
		add_action( 'admin_footer', array( $this, 'custom_bulk_admin_footer') );
		add_action( 'load-upload.php', array( $this, 'custom_bulk_action') );
		add_action( 'admin_notices', array( $this, 'custom_bulk_admin_notices') );

	}

	/* ==================================================
	 * Add a "Settings" link to the plugins page
	 * @since	1.0
	 */
	function settings_link( $links, $file ) {
		static $this_plugin;
		if ( empty($this_plugin) ) {
			$this_plugin = 'exif-caption/exifcaption.php';
		}
		if ( $file == $this_plugin ) {
			$links[] = '<a href="'.admin_url('upload.php?page=exifcaption-settings').'">'.__('Settings').'</a>';
		}
			return $links;
	}

	/* ==================================================
	 * Settings page
	 * @since	1.0
	 */
	function add_pages() {
		add_media_page(
				'Exif Caption'.' '.__('Settings'),
				'Exif Caption'.' '.__('Settings'),
				'upload_files',
				'exifcaption-settings',
				array($this, 'settings_page')
		);
	}

	/* ==================================================
	 * Add Css and Script
	 * @since	1.0
	 */
	function load_custom_wp_admin_style() {
		if ($this->is_my_plugin_screen()) {
			wp_enqueue_style( 'jquery-responsiveTabs', plugin_dir_url( __DIR__ ).'css/responsive-tabs.css' );
			wp_enqueue_style( 'jquery-responsiveTabs-style', plugin_dir_url( __DIR__ ).'css/style.css' );
			wp_enqueue_script( 'jquery' );
			wp_enqueue_script( 'jquery-responsiveTabs', plugin_dir_url( __DIR__ ).'js/jquery.responsiveTabs.min.js' );
			wp_enqueue_script( 'exifcaption-js', plugin_dir_url( __DIR__ ).'js/jquery.exifcaption.js', array('jquery') );
		}
	}

	/* ==================================================
	 * For only admin style
	 * @since	1.0
	 */
	function is_my_plugin_screen() {
		$screen = get_current_screen();
		if (is_object($screen) && $screen->id == 'media_page_exifcaption-settings') {
			return TRUE;
		} else {
			return FALSE;
		}
	}

	/* ==================================================
	 * Sub Menu
	 */
	function settings_page() {

		if ( !current_user_can( 'upload_files' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}

		if( !empty($_POST) ) {
			$post_nonce_field = 'exifcaption_tabs';
			if ( isset($_POST[$post_nonce_field]) && $_POST[$post_nonce_field] ) {
				if ( check_admin_referer( 'efc_settings', $post_nonce_field ) ) {
					$this->options_updated();
				}
			}
		}

		$user = wp_get_current_user();
		$userid = $user->ID;
		$wp_options_name = 'exifcaption_settings'.'_'.$userid;
		$exifcaption_settings = get_option($wp_options_name);
		$scriptname = admin_url('upload.php?page=exifcaption-settings');

		?>
		<div class="wrap">

		<h2>Exif Caption</h2>

		<div id="exifcaption-admin-tabs">
		  	<ul>
				<li><a href="#exifcaption-admin-tabs-1"><?php _e('Settings'); ?></a></li>
				<li><a href="#exifcaption-admin-tabs-2"><?php _e('Donate to this plugin &#187;'); ?></a></li>
			</ul>
			<div id="exifcaption-admin-tabs-1">
				<h2><?php _e('Settings'); ?></h2>
				<form method="post" action="<?php echo $scriptname; ?>">
					<?php wp_nonce_field('efc_settings', 'exifcaption_tabs'); ?>

					<div style="width: 100%; height: 100%; float: left;	margin: 5px; padding: 5px; border: #CCC 2px solid;">
						<h3><?php _e('Insertion into the caption.', 'exif-caption'); ?></h3>
						<div style="display: block; padding:5px 5px;">
						<input type="radio" name="exifcaption_insert" value="overwrite" <?php if ($exifcaption_settings['caption_insert'] === 'overwrite') echo 'checked'; ?>>
						<?php _e('Overwrite', 'exif-caption'); ?>
						</div>
						<div style="display: block; padding:5px 5px;">
						<input type="radio" name="exifcaption_insert" value="left" <?php if ($exifcaption_settings['caption_insert'] === 'left') echo 'checked'; ?>>
						<?php _e('Insert to left.', 'exif-caption'); ?>
						</div>
						<div style="display: block; padding:5px 5px;">
						<input type="radio" name="exifcaption_insert" value="right" <?php if ($exifcaption_settings['caption_insert'] === 'right') echo 'checked'; ?>>
						<?php _e('Insert to right.', 'exif-caption'); ?>
						</div>
						<div style="display: block; padding:5px 20px;">
							Exif <?php _e('Tags'); ?>
							<input type="submit" style="position:relative; top:-5px;" class="button" name="ExifDefault" value="<?php _e('Default') ?>" />
							<div style="display: block; padding:5px 20px;">
							<textarea name="exifcaption_exif_text" style="width: 100%;"><?php echo $exifcaption_settings['exif_text']; ?></textarea>
								<div>
								<a href="https://codex.wordpress.org/Function_Reference/wp_read_image_metadata#Return%20Values" target="_blank" style="text-decoration: none; word-break: break-all;"><?php _e('For Exif tags, please read here.', 'exif-caption'); ?></a>
								</div>
								<div style="display: block; padding:5px 20px;">
								<?php echo __('Extend', 'exif-caption').' '.__('Tags'); ?>
									<div>
									<b>%white_balance%</b> : <?php echo __('White balance mode(auto or manual).', 'exif-caption'); ?>
									</div>
								</div>
							</div>
						</div>
						<div>
						<?php
							if ( is_multisite() ) {
								$extendmediaupload_install_url = network_admin_url('plugin-install.php?tab=plugin-information&plugin=extend-media-upload');
								$mediafromftp_install_url = network_admin_url('plugin-install.php?tab=plugin-information&plugin=media-from-ftp');
							} else {
								$extendmediaupload_install_url = admin_url('plugin-install.php?tab=plugin-information&plugin=extend-media-upload');
								$mediafromftp_install_url = admin_url('plugin-install.php?tab=plugin-information&plugin=media-from-ftp');
							}
							$extendmediaupload_install_html = '<a href="'.$extendmediaupload_install_url.'" target="_blank" style="text-decoration: none; word-break: break-all;">Extend Media Upload</a>';
							$mediafromftp_install_html = '<a href="'.$mediafromftp_install_url.'" target="_blank" style="text-decoration: none; word-break: break-all;">Media from FTP</a>';
							echo sprintf(__('If you want to insert the Exif at the media registration, Please use the %1$s or %2$s.','exif-caption'), $extendmediaupload_install_html, $mediafromftp_install_html);
						?>
						</div>
					</div>
				<?php submit_button( __('Save Changes'), 'large', 'Submit', FALSE ); ?>
				</form>
			</div>
			<div id="exifcaption-admin-tabs-2">
			<?php $this->credit(); ?>
			</div>
		</div>

		</div>
		<?php
	}

	/* ==================================================
	 * Credit
	 */
	function credit() {

		$plugin_name = NULL;
		$plugin_ver_num = NULL;
		$plugin_path = plugin_dir_path( __DIR__ );
		$plugin_dir = untrailingslashit($plugin_path);
		$slugs = explode('/', $plugin_dir);
		$slug = end($slugs);
		$files = scandir($plugin_dir);
		foreach ($files as $file) {
			if($file == '.' || $file == '..' || is_dir($plugin_path.$file)){
				continue;
			} else {
				$exts = explode('.', $file);
				$ext = strtolower(end($exts));
				if ( $ext === 'php' ) {
					$plugin_datas = get_file_data( $plugin_path.$file, array('name'=>'Plugin Name', 'version' => 'Version') );
					if ( array_key_exists( "name", $plugin_datas ) && !empty($plugin_datas['name']) && array_key_exists( "version", $plugin_datas ) && !empty($plugin_datas['version']) ) {
						$plugin_name = $plugin_datas['name'];
						$plugin_ver_num = $plugin_datas['version'];
						break;
					}
				}
			}
		}
		$plugin_version = __('Version:').' '.$plugin_ver_num;
		$faq = __('https://wordpress.org/plugins/'.$slug.'/faq', $slug);
		$support = 'https://wordpress.org/support/plugin/'.$slug;
		$review = 'https://wordpress.org/support/view/plugin-reviews/'.$slug;
		$translate = 'https://translate.wordpress.org/projects/wp-plugins/'.$slug;
		$facebook = 'https://www.facebook.com/katsushikawamori/';
		$twitter = 'https://twitter.com/dodesyo312';
		$youtube = 'https://www.youtube.com/channel/UC5zTLeyROkvZm86OgNRcb_w';
		$donate = __('https://riverforest-wp.info/donate/', $slug);

		?>
		<span style="font-weight: bold;">
		<div>
		<?php echo $plugin_version; ?> | 
		<a style="text-decoration: none;" href="<?php echo $faq; ?>" target="_blank"><?php _e('FAQ'); ?></a> | <a style="text-decoration: none;" href="<?php echo $support; ?>" target="_blank"><?php _e('Support Forums'); ?></a> | <a style="text-decoration: none;" href="<?php echo $review; ?>" target="_blank"><?php _e('Reviews', 'media-from-ftp'); ?></a>
		</div>
		<div>
		<a style="text-decoration: none;" href="<?php echo $translate; ?>" target="_blank"><?php echo sprintf(__('Translations for %s'), $plugin_name); ?></a> | <a style="text-decoration: none;" href="<?php echo $facebook; ?>" target="_blank"><span class="dashicons dashicons-facebook"></span></a> | <a style="text-decoration: none;" href="<?php echo $twitter; ?>" target="_blank"><span class="dashicons dashicons-twitter"></span></a> | <a style="text-decoration: none;" href="<?php echo $youtube; ?>" target="_blank"><span class="dashicons dashicons-video-alt3"></span></a>
		</div>
		</span>

		<div style="width: 250px; height: 180px; margin: 5px; padding: 5px; border: #CCC 2px solid;">
		<h3><?php _e('Please make a donation if you like my work or would like to further the development of this plugin.', $slug); ?></h3>
		<div style="text-align: right; margin: 5px; padding: 5px;"><span style="padding: 3px; color: #ffffff; background-color: #008000">Plugin Author</span> <span style="font-weight: bold;">Katsushi Kawamori</span></div>
		<button type="button" style="margin: 5px; padding: 5px;" onclick="window.open('<?php echo $donate; ?>')"><?php _e('Donate to this plugin &#187;'); ?></button>
		</div>

		<?php

	}

	/* ==================================================
	 * Update	wp_options table.
	 * @since	1.0
	 */
	function options_updated(){

		$user = wp_get_current_user();
		$userid = $user->ID;
		$wp_options_name = 'exifcaption_settings'.'_'.$userid;

		$exifcaption_settings = get_option($wp_options_name);

		if ( !empty($_POST) ) {
			if ( !empty($_POST['exifcaption_insert']) ) {
				$caption_insert = sanitize_text_field($_POST['exifcaption_insert']);
			} else {
				$caption_insert = $exifcaption_settings['caption_insert'];
			}
			if ( !empty($_POST['exifcaption_exif_text']) ) {
				$exif_text = htmlspecialchars($_POST['exifcaption_exif_text']);
			} else {
				$exif_text = $exifcaption_settings['exif_text'];
			}
			if ( !empty($_POST['ExifDefault']) ) {
				$exif_text = '%title% %credit% %camera% %caption% %created_timestamp% %copyright% %aperture% %shutter_speed% %iso% %focal_length% %white_balance%';
			}
			$exifcaption_tbl = array(
								'caption_insert' => $caption_insert,
								'exif_text' => $exif_text
								);
			update_option( $wp_options_name, $exifcaption_tbl );
			echo '<div class="notice notice-success is-dismissible"><ul><li>'.__('Settings').' --> '.__('Changes saved.').'</li></ul></div>';
		}

	}

	/* ==================================================
	 * Media Library Column
	 * @param	array	$cols
	 * @return	array	$cols
	 * @since	2.0
	 */
	function muc_column( $cols ) {

		global $pagenow;
		if($pagenow == 'upload.php') {
			$cols["media_caption"] = __('Caption');
		}

		return $cols;
	}

	/* ==================================================
	 * Media Library Column
	 * @param	string	$column_name
	 * @param	int		$id
	 * @since	2.0
	 */
	function muc_value( $column_name, $id ) {

		if ( $column_name == "media_caption" ) {

			$user = wp_get_current_user();
			$userid = $user->ID;
			$wp_options_name = 'exifcaption_settings'.'_'.$userid;
			$exifcaption_settings = get_option($wp_options_name);
			$caption_insert = $exifcaption_settings['caption_insert'];
			$exif_text_tag = $exifcaption_settings['exif_text'];

			global $wpdb;
			$attachments = $wpdb->get_results($wpdb->prepare("
						SELECT	post_excerpt
						FROM	$wpdb->posts
						WHERE	ID = %d
						",$id
						),ARRAY_A);

			$exifcaption = new ExifCaption();

			$exif_text = $exifcaption->getmeta($id, $exif_text_tag);
			$input_html = NULL;
			if ( $exif_text ) {
				switch ($caption_insert) {
					case 'overwrite':
						$caption = $exif_text;
						break;
					case 'left':
						$caption = $exif_text.$attachments[0]['post_excerpt'];
						break;
					case 'right':
						$caption = $attachments[0]['post_excerpt'].$exif_text;
						break;
				}

				$input_html .= '<div>'.'['.__('Current', 'exif-caption').']: '.$attachments[0]['post_excerpt'];
				$input_html .= '</div>';
				$input_html .= '<div>'.'['.__('After', 'exif-caption').']:';
				$input_html .= '<textarea name="exifcaptiontexts['.$id.']" style="width: 100%">'.$caption.'</textarea>';
				$input_html .= '</div>';
				$input_html .= '</div>';

			}
			unset($exifcaption);
			echo $input_html;

		}

	}

	/* ==================================================
	 * Bulk Action Select
	 * @since	2.0
	 */
	function custom_bulk_admin_footer() {
		global $pagenow;
		if($pagenow == 'upload.php') {
			?>
			<script type="text/javascript">
				jQuery(document).ready(function() {
					jQuery('<option>').val('insertexifcaption').text('<?php _e('Insert Exif to Caption', 'exif-caption')?>').appendTo("select[name='action']");
					jQuery('<option>').val('insertexifcaption').text('<?php _e('Insert Exif to Caption', 'exif-caption')?>').appendTo("select[name='action2']");
				});
			</script>
			<?php
		}
	}

	/* ==================================================
	 * Bulk Action
	 * @since	2.0
	 */
	function custom_bulk_action() {

		if ( !isset( $_REQUEST['detached'] ) ) {

			// get the action
			$wp_list_table = _get_list_table('WP_Media_List_Table');  
			$action = $wp_list_table->current_action();

			$allowed_actions = array("insertexifcaption");
			if(!in_array($action, $allowed_actions)) return;

			check_admin_referer('bulk-media');

			if(isset($_REQUEST['media'])) {
				$post_ids = array_map('intval', $_REQUEST['media']);
			}

			if(empty($post_ids)) return;

			$sendback = remove_query_arg( array('exifinserted', 'message', 'untrashed', 'deleted', 'ids'), wp_get_referer() );
			if ( ! $sendback )
			$sendback = admin_url( "upload.php?post_type=$post_type" );

			$pagenum = $wp_list_table->get_pagenum();
			$sendback = add_query_arg( 'paged', $pagenum, $sendback );

			switch($action) {
				case 'insertexifcaption':
					$exifinserted = 0;
					$exifcaption_texts = $this->sanitize_array($_REQUEST['exifcaptiontexts']);

					$exifcaption = new ExifCaption();

					global $wpdb;
					$messages = array();
					foreach( $post_ids as $post_id ) {
						// Change DB Attachement post
						$update_array = array(
										'post_excerpt'=> $exifcaption_texts[$post_id]
									);
						$id_array= array('ID'=> $post_id);
						$wpdb->update( $wpdb->posts, $update_array, $id_array, array('%s'), array('%d') );
						unset($update_array, $id_array);

						$message = $exifcaption->replace_content_caption($post_id, $exifcaption_texts[$post_id]);

						if ( $message ) {
							$messages[$exifinserted] = $message;
							$exifinserted++;
						}
					}
					unset($exifcaption);
					$sendback = add_query_arg( array('exifinserted' => $exifinserted, 'ids' => join(',', $post_ids), 'message' => join(',',  $messages)), $sendback );
				break;
				default: return;
			}

			$sendback = remove_query_arg( array('action', 'action2', 'tags_input', 'post_author', 'comment_status', 'ping_status', '_status',  'post', 'bulk_edit', 'post_view'), $sendback );
			wp_redirect($sendback);
			exit();

		}

	}

	/* ==================================================
	 * Bulk Action Message
	 * @since	2.0
	 */
	function custom_bulk_admin_notices() {

	    global $post_type, $pagenow;

		if ( $pagenow == 'upload.php' && $post_type == 'attachment' && isset($_REQUEST['exifinserted']) && intval($_REQUEST['exifinserted']) > 0 && isset($_REQUEST['message']) ) {
			$messages = explode(',', sanitize_text_field(urldecode($_REQUEST['message'])) );
			$success_count = 0;
			foreach ( $messages as $message ) {
				if ( $message === 'success' ) {
					++$success_count;
				} else {
					echo '<div class="notice notice-error is-dismissible"><ul><li>'.$message.'</li></ul></div>';
				}
			}
			if ( $success_count > 0 ) {
				echo '<div class="notice notice-success is-dismissible"><ul><li>'.sprintf(__('%1$d media files updated.', 'exif-caption'),$success_count).'</li></ul></div>';
			}
		}

	}

	/* ==================================================
	* Sanitize Array
	* @param	array	$a
	* @return	string	$_a
	* @since	2.04
	*/
	function sanitize_array($a) {

		$_a = array();
		foreach($a as $key=>$value) {
			if ( is_array($value) ) {
				$_a[$key] = $this->sanitize_array($value);
			} else {
				$_a[$key] = htmlspecialchars($value);
			}
		}

		return $_a;

	}

}

?>